# Танец Феи Драже
# Подключаем библиотеки
import fluidsynth
from pyiArduinoI2Cbumper import *
from pyiArduinoI2Cmotor import *
from pyiArduinoI2Cencoder import *
from time import sleep

# Адреса модулей i2c
BUM_ADDRESS = 0x0a
MOT_ADDRESS = 0x0b
ENC_ADDRESS = 0x0c

# Диапазон энкодера
ENC_RANGE = 2

#Начальная скорость мотора (ШИМ)
MOT_SPEED = 8

# Тембр инструмента MIDI
# (10 - музыкальная шкатулка)
# Полный список названий General Midi звуков -
# https://soundprogramming.net/file-formats/general-midi-instrument-list/
MUSIC_BOX = 10

# Константа дорожки инструмента
TRACK = 0

# Динамика по умолчанию
DEF_VEL = 100

# Создаём объект бампера
bum = pyiArduinoI2Cbumper(BUM_ADDRESS)

# Создаём объект мотора
mot = pyiArduinoI2Cmotor(MOT_ADDRESS)

# Создаём объект энкодера
enc = pyiArduinoI2Cencoder(ENC_ADDRESS)
enc.setPosSettings(ENC_RANGE)

# Создаём объект синтезатора
synth = fluidsynth.Synth()

# Включаем синтезатор, указывая драйвер
synth.start(driver='alsa')

# Подключаем файл звукового шрифта.
# Если файл шрифта не лежит в одной папке
# с этим скриптом, укажите абсолютный путь здесь
font = synth.sfload('soundfont.sf2')

# Выбираем программу синтезатора
synth.program_select(0, font, 0, MUSIC_BOX)

# Гамма для проигрывания (в данном случае пентатоника Ля-минор)
# Цифры - ноты по номенклатуре MIDI, где 60 - нота ДО малой октавы
scale = [ 91, 90, 88, 87, 86, 85, 84, 83, 81 ]

# Список проигрываемых нот
is_playing = [False] * len(scale)

try:
    # Запускаем мотор
    mot.setSpeed(MOT_SPEED, MOT_PWM)


    # Читаем положение энкодера
    p = enc.getPosition()

    while True:

        # Читаем положение энкодера в бесконечном цикле
        new_p = enc.getPosition()

        # Если значение отличное от предыдущего...
        if p != new_p:

            p = new_p

            # ...меняем скорость вращения мотора
            mot.setSpeed(5 + p, MOT_PWM)

        # Читаем состояние всех датчиков линий
        readall = bum.getLineDigital(BUM_LINE_ALL)

        for i in range(len(scale)):

            # Если бит датчика установлен
            if readall & (1 << i + 1):

                # и нота не проигрывается в данным момент
                if is_playing[i] == False:

                    # Включаем соответствующую ноту синтезатора
                    synth.noteon(TRACK, scale[i], DEF_VEL)
                    is_playing[i] = True

            else:
                if is_playing[i] == True:
                    synth.noteoff(TRACK, scale[i])
                    is_playing[i] = False

# Если выполнения скрипта прервано - посылаем сигнал закончить все ноты
# и остановить мотор
except:

    for i in range(127):
        synth.noteoff(TRACK, i)

    mot.setSpeed(0, MOT_PWM)

