# Дым над водой (Smoke On The Water)
# Подключаем библиотеки
import fluidsynth
from pyiArduinoI2Cbumper import *
from pyiArduinoI2Cmotor import *
from pyiArduinoI2Cencoder import *
from time import sleep

# Адреса модулей i2c
BUM_ADDRESS = 0x0a
MOT_ADDRESS = 0x0b
ENC_ADDRESS = 0x0c

# Диапазон энкодера
ENC_RANGE = 2

#Начальная скорость мотора (ШИМ)
MOT_SPEED = 10

# Константы с номерами инструментов
ROCK_ORGAN = 18
BASS = 35
BANK = 0
DRUM_BANK = 128
DRUMS = 0

# Константы дорожек инструментов
BASS_TRACK = 0
ORGAN_TRACK = 1
DRUM_TRACK = 10

# Констанаты разделения инструментов по датчикам
SPLIT_1 = 4
SPLIT_2 = 7

# Динамика по умолчанию
DEF_VEL = 100

# Создаём объект бампера
bum = pyiArduinoI2Cbumper(BUM_ADDRESS)

# Создаём объект мотора
mot = pyiArduinoI2Cmotor(MOT_ADDRESS)

# Создаём объект мотора
enc = pyiArduinoI2Cencoder(ENC_ADDRESS)

# Устанавливаем диапазон энкодера
enc.setPosSettings(ENC_RANGE)

# Создаём объект синтезатора
synth = fluidsynth.Synth()

# Включаем синтезатор, указывая драйвер
synth.start(driver='alsa')

# Подключаем файл звукового шрифта.
# Если файл шрифта не лежит в одной папке
# с этим скриптом, укажите абсолютный путь здесь
font = synth.sfload('soundfont.sf2')

# Выбираем программу синтезатора для каждого инструмента
# Полный список названий General Midi звуков -
# https://soundprogramming.net/file-formats/general-midi-instrument-list/
synth.program_select(BASS_TRACK, font, BANK, BASS)
synth.program_select(ORGAN_TRACK, font, BANK, ROCK_ORGAN)
synth.program_select(DRUM_TRACK, font, DRUM_BANK, DRUMS)

# Гамма для проигрывания
# Цифры - ноты по номенклатуре MIDI, где 60 - нота ДО малой октавы
scale = [ 73, 72, 70, 67, 36, 34, 31, 38, 36 ]

# Список проигрываемых нот
is_playing = [False] * len(scale)

try:
    # Запускаем мотор
    mot.setSpeed(MOT_SPEED, MOT_PWM)

    # Считываем положение энкодера
    p = enc.getPosition()

    while True:

        # Считываем положение энкодера в бесконечном цикле
        new_p = enc.getPosition()

        # Если положение энкодера изменилось:
        if p != new_p:

            p = new_p
            # меняем скорость мотора
            mot.setSpeed(5 + p, MOT_PWM)


        # Читаем состояние всех датчиков линий
        readall = bum.getLineDigital(BUM_LINE_ALL)

        for i in range(len(scale)):

            # Если бит датчика установлен
            if readall & (1 << i + 1):

                track = 0

                # Выбор инструмента
                if i < SPLIT_1:

                    # Орган
                    track = ORGAN_TRACK

                elif i < SPLIT_2:

                    # Бас
                    track = BASS_TRACK

                else:

                    # Барабаны
                    track = DRUM_TRACK

                # Выбор ноты
                if is_playing[i] == False:
                    synth.noteon(track, scale[i], DEF_VEL)

                    # Если это орган - добавляем кварту снизу (-5 полутонов)
                    if track == ORGAN_TRACK:
                        synth.noteon(track, scale[i] - 5, DEF_VEL)

                    is_playing[i] = True
            else:

                track = 0

                # Выбор инструмента
                if i < SPLIT_1:

                    # Орган
                    track = ORGAN_TRACK

                elif i < SPLIT_2:

                    # Бас
                    track = BASS_TRACK

                else:

                    # Барабаны
                    track = DRUM_TRACK

                # Выбор ноты
                if is_playing[i] == True:
                    synth.noteoff(track, scale[i])

                    if track == ORGAN_TRACK:
                        synth.noteoff(track, scale[i] - 5)

                    is_playing[i] = False

# Если выполнения скрипта прервано - посылаем сигнал закончить все ноты
# и остановить мотор
except:

    for i in range(127):
        synth.noteoff(DRUM_TRACK, i)
        synth.noteoff(BASS_TRACK, i)
        synth.noteoff(ORGAN_TRACK, i)

    mot.setSpeed(0, MOT_PWM)
